searchNodes=[{"doc":"CubDB is an embedded key-value database written in the Elixir language. It runs locally, it is schema-less, and backed by a single file.FeaturesBoth keys and values can be any arbitrary Elixir (or Erlang) term.Simple get/3, put/3, and delete/2 operationsArbitrary selection of ranges of entries sorted by key with select/2Atomic transactions with put_multi/2, get_and_update_multi/4, etc.Concurrent read operations, that do not block nor are blocked by writesUnexpected shutdowns won't corrupt the database or break atomicityManual or automatic compaction to optimize space usageTo ensure consistency, performance, and robustness to data corruption, CubDB database file uses an append-only, immutable B-tree data structure. Entries are never changed in-place, and read operations are performend on immutable snapshots.More information can be found in the following sections:Frequently Asked QuestionsHow ToUsageStart CubDB by specifying a directory for its database file (if not existing, it will be created):{:ok, db} = CubDB.start_link(&quot;my/data/directory&quot;)Alternatively, to specify more options, a keyword list can be passed:{:ok, db} = CubDB.start_link(data_dir: &quot;my/data/directory&quot;, auto_compact: true)Important: avoid starting multiple CubDB processes on the same data directory. Only one CubDB process should use a specific data directory at any time.CubDB functions can be called concurrently from different processes, but it is important that only one CubDB process is started on the same data directory.The get/2, put/3, and delete/2 functions work as you probably expect:CubDB.put(db, :foo, &quot;some value&quot;) #=&gt; :ok CubDB.get(db, :foo) #=&gt; &quot;some value&quot; CubDB.delete(db, :foo) #=&gt; :ok CubDB.get(db, :foo) #=&gt; nilBoth keys and values can be any Elixir (or Erlang) term:CubDB.put(db, {&quot;some&quot;, &#39;tuple&#39;, :key}, %{foo: &quot;a map value&quot;}) #=&gt; :ok CubDB.get(db, {&quot;some&quot;, &#39;tuple&#39;, :key}) #=&gt; %{foo: &quot;a map value&quot;}Multiple operations can be performed as an atomic transaction with put_multi/2, delete_multi/2, and the other [...]_multi functions:CubDB.put_multi(db, [a: 1, b: 2, c: 3, d: 4, e: 5, f: 6, g: 7, h: 8]) #=&gt; :okRange of entries sorted by key are retrieved using select/2:CubDB.select(db, min_key: :b, max_key: :e) #=&gt; {:ok, [b: 2, c: 3, d: 4, e: 5]}But select/2 can do much more than that. It can apply a pipeline of operations (map, filter, take, drop and more) to the selected entries, it can select the entries in normal or reverse order, and it can reduce the result using an arbitrary function:# Take the sum of the last 3 even values: CubDB.select(db, # select entries in reverse order reverse: true, # apply a pipeline of operations to the entries pipe: [ # map each entry discarding the key and keeping only the value map: fn {_key, value} -&gt; value end, # filter only even integers filter: fn value -&gt; is_integer(value) &amp;&amp; Integer.is_even(value) end, # take the first 3 values take: 3 ], # reduce the result to a sum reduce: fn n, sum -&gt; sum + n end ) #=&gt; {:ok, 18}Because CubDB uses an immutable data structure, write operations cause the data file to grow. When necessary, CubDB runs a compaction operation to optimize the file size and reclaim disk space. Compaction runs in the background, without blocking other operations. By default, CubDB runs compaction automatically when necessary (see documentation of set_auto_compact/2 for details). Alternatively, it can be started manually by calling compact/1.","ref":"CubDB.html","title":"CubDB","type":"module"},{"doc":"","ref":"CubDB.html#catch_up/3","title":"CubDB.catch_up/3","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.The default options listed in Supervisor are used.","ref":"CubDB.html#child_spec/1","title":"CubDB.child_spec/1","type":"function"},{"doc":"Runs a database compaction.As write operations are performed on a database, its file grows. Occasionally, a compaction operation can be run to shrink the file to its optimal size. Compaction runs in the background and does not block operations.Only one compaction operation can run at any time, therefore if this function is called when a compaction is already running, it returns {:error, :pending_compaction}.When compacting, CubDB will create a new data file, and eventually switch to it and remove the old one as the compaction succeeds. For this reason, during a compaction, there should be enough disk space for a second copy of the database file.Compaction can create disk contention, so it should not be performed unnecessarily often.","ref":"CubDB.html#compact/1","title":"CubDB.compact/1","type":"function"},{"doc":"Returns the path of the current database file.The current database file will change after a compaction operation.Example{:ok, db} = CubDB.start_link(&quot;some/data/directory&quot;) CubDB.current_db_file(db) #=&gt; &quot;some/data/directory/0.cub&quot;","ref":"CubDB.html#current_db_file/1","title":"CubDB.current_db_file/1","type":"function"},{"doc":"Returns the path of the data directory, as given when the CubDB process was started.Example{:ok, db} = CubDB.start_link(&quot;some/data/directory&quot;) CubDB.data_dir(db) #=&gt; &quot;some/data/directory&quot;","ref":"CubDB.html#data_dir/1","title":"CubDB.data_dir/1","type":"function"},{"doc":"Deletes the entry associated to key from the database.If key was not present in the database, nothing is done.","ref":"CubDB.html#delete/2","title":"CubDB.delete/2","type":"function"},{"doc":"Deletes multiple entries corresponding to the given keys all at once, atomically.The keys to be deleted are passed as a list.","ref":"CubDB.html#delete_multi/2","title":"CubDB.delete_multi/2","type":"function"},{"doc":"Returns the dirt factor.The dirt factor is a number, ranging from 0 to 1, giving an indication about the amount of overhead disk space (or &quot;dirt&quot;) that can be cleaned up with a compaction operation. A value of 0 means that there is no overhead, so a compaction would have no benefit. The closer to 1 the dirt factor is, the more can be cleaned up in a compaction operation.","ref":"CubDB.html#dirt_factor/1","title":"CubDB.dirt_factor/1","type":"function"},{"doc":"Fetches the value for the given key in the database, or return :error if key is not present.If the database contains an entry with the given key and value value, it returns {:ok, value}. If key is not found, it returns :error.","ref":"CubDB.html#fetch/2","title":"CubDB.fetch/2","type":"function"},{"doc":"Performs a fsync, forcing to flush all data that might be buffered by the OS to disk.Calling this function ensures that all writes up to this point are committed to disk, and will be available after a restart.If CubDB is started with the option auto_file_sync: true, calling this function is not necessary, as every write operation will be automatically flushed to the storage device.If this function is NOT called, the operative system will control when the file buffer is flushed to the storage device, which leads to better write performance, but might affect durability of recent writes in case of a sudden shutdown.","ref":"CubDB.html#file_sync/1","title":"CubDB.file_sync/1","type":"function"},{"doc":"Gets the value associated to key from the database.If no value is associated with key, default is returned (which is nil, unless specified otherwise).","ref":"CubDB.html#get/3","title":"CubDB.get/3","type":"function"},{"doc":"Gets the value corresponding to key and updates it, in one atomic transaction.fun is called with the current value associated to key (or nil if not present), and must return a two element tuple: the result value to be returned, and the new value to be associated to key. fun may also return :pop, in which case the current value is deleted and returned.The return value is {:ok, result}, or {:error, reason} in case an error occurs.Note that in case the value to update returned by fun is the same as the original value, no write is performed to disk.","ref":"CubDB.html#get_and_update/3","title":"CubDB.get_and_update/3","type":"function"},{"doc":"Gets and updates or deletes multiple entries in an atomic transaction.Gets all values associated with keys in keys_to_get, and passes them as a map of %{key =&gt; value} entries to fun. If a key is not found, it won't be added to the map passed to fun. Updates the database and returns a result according to the return value of fun. Returns {:ok, return_value} in case of success, {:error, reason} otherwise.The function fun should return a tuple of three elements: {return_value, entries_to_put, keys_to_delete}, where return_value is an arbitrary value to be returned, entries_to_put is a map of %{key =&gt; value} entries to be written to the database, and keys_to_delete is a list of keys to be deleted.The read and write operations are executed as an atomic transaction, so they will either all succeed, or all fail. Note that get_and_update_multi/4 blocks other write operations until it completes.The options argument is an optional keyword list of options, including::timeout - a timeout (in milliseconds or :infinity, defaulting to 5000) for the operation, after which the function returns {:error, :timeout}. This is useful to avoid blocking other write operations for too long.ExampleAssuming a database of names as keys, and integer monetary balances as values, and we want to transfer 10 units from &quot;Anna&quot; to &quot;Joy&quot;, returning their updated balance:{:ok, {anna, joy}} = CubDB.get_and_update_multi(db, [&quot;Anna&quot;, &quot;Joy&quot;], fn entries -&gt; anna = Map.get(entries, &quot;Anna&quot;, 0) joy = Map.get(entries, &quot;Joy&quot;, 0) if anna &lt; 10, do: raise(RuntimeError, message: &quot;Anna&#39;s balance is too low&quot;) anna = anna - 10 joy = joy + 10 {{anna, joy}, %{&quot;Anna&quot; =&gt; anna, &quot;Joy&quot; =&gt; joy}, []} end)Or, if we want to transfer all of the balance from &quot;Anna&quot; to &quot;Joy&quot;, deleting &quot;Anna&quot;'s entry, and returning &quot;Joy&quot;'s resulting balance:{:ok, joy} = CubDB.get_and_update_multi(db, [&quot;Anna&quot;, &quot;Joy&quot;], fn entries -&gt; anna = Map.get(entries, &quot;Anna&quot;, 0) joy = Map.get(entries, &quot;Joy&quot;, 0) joy = joy + anna {joy, %{&quot;Joy&quot; =&gt; joy}, [&quot;Anna&quot;]} end)","ref":"CubDB.html#get_and_update_multi/4","title":"CubDB.get_and_update_multi/4","type":"function"},{"doc":"Gets multiple entries corresponding by the given keys all at once, atomically.The keys to get are passed as a list. The result is a map of key/value entries corresponding to the given keys. Keys that are not present in the database won't be in the result map.ExampleCubDB.put_multi(db, a: 1, b: 2, c: nil) CubDB.get_multi(db, [:a, :b, :c, :x]) # =&gt; %{a: 1, b: 2, c: nil}","ref":"CubDB.html#get_multi/2","title":"CubDB.get_multi/2","type":"function"},{"doc":"Returns whether an entry with the given key exists in the database.","ref":"CubDB.html#has_key?/2","title":"CubDB.has_key?/2","type":"function"},{"doc":"Writes an entry in the database, associating key to value.If key was already present, it is overwritten.","ref":"CubDB.html#put/3","title":"CubDB.put/3","type":"function"},{"doc":"Writes and deletes multiple entries all at once, atomically.Entries to put are passed as a map of %{key =&gt; value} or a list of {key, value}. Keys to delete are passed as a list of keys.","ref":"CubDB.html#put_and_delete_multi/3","title":"CubDB.put_and_delete_multi/3","type":"function"},{"doc":"Writes multiple entries all at once, atomically.Entries are passed as a map of %{key =&gt; value} or a list of {key, value}.","ref":"CubDB.html#put_multi/2","title":"CubDB.put_multi/2","type":"function"},{"doc":"Writes an entry in the database, associating key to value, only if key is not yet in the database.If key is already present, it does not change it, and returns {:error, :exists}.","ref":"CubDB.html#put_new/3","title":"CubDB.put_new/3","type":"function"},{"doc":"Selects a range of entries from the database, and optionally performs a pipeline of operations on them.It returns {:ok, result} if successful, or {:error, exception} if an exception is raised.OptionsThe min_key and max_key specify the range of entries that are selected. By default, the range is inclusive, so all entries that have a key greater or equal than min_key and less or equal then max_key are selected:# Select all entries where &quot;a&quot; &lt;= key &lt;= &quot;d&quot; CubDB.select(db, min_key: &quot;b&quot;, max_key: &quot;d&quot;)The range boundaries can be excluded by setting min_key_inclusive or max_key_inclusive to false:# Select all entries where &quot;a&quot; &lt;= key &lt; &quot;d&quot; CubDB.select(db, min_key: &quot;b&quot;, max_key: &quot;d&quot;, max_key_inclusive: false)Any of :min_key and :max_key can be omitted, to leave the range open-ended.# Select entries where key &lt;= &quot;a&quot; CubDB.select(db, max_key: &quot;a&quot;)As nil is a valid key, setting min_key or max_key to nil does NOT leave the range open ended:# Select entries where nil &lt;= key &lt;= &quot;a&quot; CubDB.select(db, min_key: nil, max_key: &quot;a&quot;)The reverse option, when set to true, causes the entries to be selected and traversed in reverse order.The pipe option specifies an optional list of operations performed sequentially on the selected entries. The given order of operations is respected. The available operations, specified as tuples, are:{:filter, fun} filters entries for which fun returns a truthy value{:map, fun} maps each entry to the value returned by the function fun{:take, n} takes the first n entries{:drop, n} skips the first n entries{:take_while, fun} takes entries while fun returns a truthy value{:drop_while, fun} skips entries while fun returns a truthy valueNote that, when selecting a key range, specifying min_key and/or max_key is more performant than using {:filter, fun} or {:take_while | :drop_while, fun}, because min_key and max_key avoid loading unnecessary entries from disk entirely.The reduce option specifies how the selected entries are aggregated. If reduce is omitted, the entries are returned as a list. If reduce is a function, it is used to reduce the collection of entries. If reduce is a tuple, the first element is the starting value of the reduction, and the second is the reducing function.The timeout option specifies a timeout (in milliseconds or :infinity, defaulting to :infinity) after which the operation will fail.ExamplesTo select all entries with keys between :a and :c as a list of {key, value} entries we can do:{:ok, entries} = CubDB.select(db, min_key: :a, max_key: :c)If we want to get all entries with keys between :a and :c, with :c excluded, we can do:{:ok, entries} = CubDB.select(db, min_key: :a, max_key: :c, max_key_inclusive: false)To select the last 3 entries, we can do:{:ok, entries} = CubDB.select(db, reverse: true, pipe: [take: 3])If we want to obtain the sum of the first 10 positive numeric values associated to keys from :a to :f, we can do:{:ok, sum} = CubDB.select(db, min_key: :a, max_key: :f, pipe: [ map: fn {_key, value} -&gt; value end, # map values filter: fn n -&gt; is_number(n) and n &gt; 0 end # only positive numbers take: 10, # take only the first 10 entries in the range ], reduce: fn n, sum -&gt; sum + n end # reduce to the sum of selected values )","ref":"CubDB.html#select/2","title":"CubDB.select/2","type":"function"},{"doc":"Configures whether to perform automatic compaction, and how.If set to false, no automatic compaction is performed. If set to true, auto-compaction is performed, following a write operation, if at least 100 write operations occurred since the last compaction, and the dirt factor is at least 0.25. These values can be customized by setting the auto_compact option to {min_writes, min_dirt_factor}.It returns :ok, or {:error, reason} if setting is invalid.Compaction is performed in the background and does not block other operations, but can create disk contention, so it should not be performed unnecessarily often. When writing a lot into the database, such as when importing data from an external source, it is adviseable to turn off auto compaction, and manually run compaction at the end of the import.","ref":"CubDB.html#set_auto_compact/2","title":"CubDB.set_auto_compact/2","type":"function"},{"doc":"Configures whether to automatically force file sync upon each write operation.If set to false, no automatic file sync is performed. That improves write performance, but leaves to the operative system the decision of when to flush disk buffers. This means that there is the possibility that recent writes might not be durable in case of a sudden machine shutdown. In any case, atomicity of multi operations is preserved, and partial writes will not corrupt the database.If set to true, the file buffer will be forced to flush upon every write operation, ensuring durability even in case of sudden machine shutdowns, but decreasing write performance.","ref":"CubDB.html#set_auto_file_sync/2","title":"CubDB.set_auto_file_sync/2","type":"function"},{"doc":"Returns the number of entries present in the database.","ref":"CubDB.html#size/1","title":"CubDB.size/1","type":"function"},{"doc":"Starts the CubDB database without a link.See start_link/2 for more information about options.","ref":"CubDB.html#start/1","title":"CubDB.start/1","type":"function"},{"doc":"","ref":"CubDB.html#start/2","title":"CubDB.start/2","type":"function"},{"doc":"Starts the CubDB database process linked to the current process.The argument is a keyword list of options:data_dir: the directory path where the database files will be stored. This option is required. If the directory does not exist, it will be created. Only one CubDB instance can run per directory, so if you run several databases, they should each use their own separate data directory.auto_compact: whether to perform compaction automatically. It defaults to true. See set_auto_compact/2 for the possible valuesauto_file_sync: whether to force flush the disk buffer on each write. It defaults to true. If set to false, write performance is faster, but durability of writes is not strictly guaranteed. See set_auto_file_sync/2 for details.GenServer options like name and timeout can also be given, and are forwarded to GenServer.start_link/3 as the third argument.If only the data_dir is specified, it is possible to pass it as a single string argument.Examples# Passing only the data dir {:ok, db} = CubDB.start_link(&quot;some/data/dir&quot;) # Passing data dir and other options {:ok, db} = CubDB.start_link(data_dir: &quot;some/data/dir&quot;, auto_compact: true, name: :db)","ref":"CubDB.html#start_link/1","title":"CubDB.start_link/1","type":"function"},{"doc":"","ref":"CubDB.html#start_link/2","title":"CubDB.start_link/2","type":"function"},{"doc":"Synchronously stops the CubDB database.See GenServer.stop/3 for details.","ref":"CubDB.html#stop/3","title":"CubDB.stop/3","type":"function"},{"doc":"Updates the entry corresponding to key using the given function.If key is present in the database, fun is invoked with the corresponding value, and the result is set as the new value of key. If key is not found, initial is inserted as the value of key.The return value is :ok, or {:error, reason} in case an error occurs.","ref":"CubDB.html#update/4","title":"CubDB.update/4","type":"function"},{"doc":"","ref":"CubDB.html#t:entry/0","title":"CubDB.entry/0","type":"type"},{"doc":"","ref":"CubDB.html#t:key/0","title":"CubDB.key/0","type":"type"},{"doc":"","ref":"CubDB.html#t:option/0","title":"CubDB.option/0","type":"type"},{"doc":"","ref":"CubDB.html#t:pipe_operation/0","title":"CubDB.pipe_operation/0","type":"type"},{"doc":"","ref":"CubDB.html#t:select_option/0","title":"CubDB.select_option/0","type":"type"},{"doc":"","ref":"CubDB.html#t:value/0","title":"CubDB.value/0","type":"type"},{"doc":"Frequently Asked Questions","ref":"faq.html","title":"Frequently Asked Questions","type":"extras"},{"doc":"If your Elixir application needs an embedded database (which is a database that run inside your application, as opposed to one that runs as a separate software), then CubDB might be a great fit: its API is simple and idiomatic, you can supervise it as any other Elixir process, and it runs wherever Elixir runs, with no need to cross-compile native code. You can think about CubDB as an Elixir collection, like a Map or List, but one that is stored to disk and persistent to restarts.The typical use-case for CubDB is data storage on single-instance applications or embedded software (for example, CubDB is a great fit for Nerves projects). In those contexts, CubDB is typically used for things like:Persisting configuration and preferencesData logging and storage of metrics or time seriesesLocal database for application data","ref":"faq.html#why-should-i-use-cubdb","title":"Frequently Asked Questions - Why should I use CubDB?","type":"extras"},{"doc":"CubDB is not jealous: it does its job well, without claiming to be better than others. There are many other great alternatives for storing data, and depending on your use-case they might be a better fit.That said, here are some reasons why you might choose CubDB over other popular alternatives:ETS (Erlang Term Store), like CubDB, can store arbitrary Elixir/Erlang terms, and comes by default with the Erlang runtime. Differently from CubDB though, ETS stores values in memory, so they are lost in case of a restart.DETS is similar to ETS, but it persists values on disk. Compared to CubDB though, it does not support sorted collections. Also, you might find its API more convoluted that CubDB.Mnesia is a distributed database system that comes with the Erlang runtime. It can be distributed, and enforces a schema. If on one hand it can make a great heavy-duty distributed database system, it is substantially more complicated to use than CubDB for embedded use cases.SQLite, LevelDB, LMDB, etc. are all great general-purpose embedded databases. They support a variety of features, and have wider adoption than CubDB. Because they are not &quot;Elixir native&quot; though, you need a library to interact with them from Elixir, and they generally feel less &quot;idiomatic&quot; than CubDB when used from an Elixir application. In some cases (like when writing software for embedded devices), it can be more complicated to install and cross-compile them for your architecture, and some libraries might not play well with your supervision strategies (NIF libraries are fast, but will crash the Erlang VM if the native executable crash).Plain files can be used to store simple data. Compared to CubDB though, they don't offer efficient key/value access or sorted collections, and they are subject to data corruption if a sudden power loss happens mid-write. CubDB is designed to be efficient and robust, so a power loss won't corrupt its data, or break atomicity.If your use case is such that one of those alternatives is a better fit, by all means go for it: CubDB won't be sad :) But if CubDB fits your use case, you will enjoy its native Elixir feel and simple but versatile model.","ref":"faq.html#how-does-it-compare-with-x","title":"Frequently Asked Questions - How does it compare with X?","type":"extras"},{"doc":"CubDB uses an append-only B-tree data structure: each change to CubDB is appended to the data file, instead of modifying the existing data in-place. This is efficient and robust: writing at the end of a file is faster than &quot;jumping around&quot;, and should something go wrong in the middle of a write (say, a power failure), no data is destroyed by a partial overwrite, so CubDB is able to recover upon restart.The drawback of this approach though, is that the data file will keep growing as you write to the database, even when you update or delete existing values. Performance of read and write operations is not affected by the file size, but space utilization can be optimized: old entries that are not &quot;reachable&quot; are still in the data file, making it larger than it needs to be. Compaction is the operation through which CubDB &quot;cleans up&quot; and makes its data file compact and space-efficient again. Other databases have similar processes (for example, SQLite calls it &quot;vacuuming&quot;).During a compaction operation, CubDB creates a new file, and transfers to it the current entries, without the stale data. When all the data is transferred, including entries written after the compaction started, CubDB switches to use the new compacted file and removes the old, obsolete one. The compaction process is performed in the background, without blocking read or write operations on the active data file.Should a compaction operation be interrupted by a shutdown or an application crash, no data is lost: the old data file is still up to date and active, and upon the next compaction the file left over by the interrupted compaction is removed and a new one created.","ref":"faq.html#what-is-compaction","title":"Frequently Asked Questions - What is compaction?","type":"extras"},{"doc":"Usually, letting CubDB perform compaction automatically is the most convenient choice: CubDB will keep track of its &quot;dirt factor&quot;, which is an indicator of how much overhead could be shaved off by a compaction, and clean up when necessary.In some situations though, it can be advisable to avoid auto compaction and compact your database manually. One example is if you are performing a one-off data import: in this case, it makes sense to import all data, and manually trigger a compaction only afterwards. This reduces disk contention during the import. Note that you can turn auto compaction on or off at runtime with CubDB.set_auto_compact/2.","ref":"faq.html#should-i-use-auto-compaction","title":"Frequently Asked Questions - Should I use auto compaction?","type":"extras"},{"doc":"When you write to a file, your operative system usually buffers writes in memory, and actually writes them in the file system only later. This makes write operations faster: file system access is expensive, and buffering batches together several writes in one single operation. The drawback is that, should a power failure happen before the buffer is written to the file system, data that was held in the buffer might be lost. When you want to make sure that data is safe in the file system, you have to tell the operative system to &quot;file sync&quot;, which means to flush the buffer to disk.With CubDB, you can chose to automatically sync each write operation, or to manually sync when you prefer. If you need faster write performance, but you are ok with a small chance of loosing recently written data in case of a power loss (for example if you use CubDB to log large quantity of data), you might choose to not perform automatic file sync. Note that the operative system will still sync data periodically, and upon clean shutdown. If instead you want to be sure that data that was successfully written by CubDB won't be lost, even in case of a power failure, you should use the auto file sync option: write operations will be sensibly slower, but each write will be safe on disk by the time the write operation completes.Even with auto file sync turned off, power failures won't corrupt the database or break atomicity. Whether to file sync or not is therefore a trade off between durability and write performance, and does not affect other semantics.","ref":"faq.html#what-does-file-sync-mean","title":"Frequently Asked Questions - What does file sync mean?","type":"extras"},{"doc":"How To","ref":"howto.html","title":"How To","type":"extras"},{"doc":"The ability of CubDB to retrieve ranges of data sorted by key makes it ideal to store sorted collections, sort of like tables in a relational database. Different collections can be stored in the same database, or in separate ones.The main reason to use separate CubDB databases is if you need different configuration options for each database (like auto compaction or file sync).Otherwise, it is usually simpler and more efficient to use a single database: read operations in CubDB execute concurrently, and while write operations on the same database are serialized, using multiple databases means losing the advantage of the append-only nature of CubDB, resulting in often slower patterns of file access.One common way to store separate collections in the same database is to use composite keys, usually tuples, leveraging the fact that the Erlang runtime defines a total ordering of all terms.Say that you want to store two collections: people and articles. You can structure your keys to be tuples like {:people, person_id} for people, and {:articles, article_id} for articles:# Add a few people: :ok = CubDB.put(db, {:people, 1}, %{first_name: &quot;Margaret&quot;, last_name: &quot;Hamilton&quot;}) :ok = CubDB.put(db, {:people, 2}, %{first_name: &quot;Alan&quot;, last_name: &quot;Turing&quot;}) # Add a few articles: :ok = CubDB.put(db, {:articles, 1}, %{title: &quot;Spaceship Guidance made easy&quot;, text: &quot;...&quot;}) :ok = CubDB.put(db, {:articles, 2}, %{title: &quot;Morphogenesis for the uninitiated&quot;, text: &quot;...&quot;})We used positive integers as IDs in our example, but you can really use anything you want.Getting a specific person or article by ID is trivial:person = CubDB.get(db, {:people, 1}) article = CubDB.get(db, {:articles, 2})Selecting all members of a collection without selecting also other collections can be easily done by leveraging the fact that tuples are compared element by element. Therefore, here is how you can select all members of a specific collection:# Select all people {:ok, people_wth_keys} = CubDB.select(db, min_key: {:people, 0}, max_key: {:people, nil}) # Select all articles {:ok, articles_with_keys} = CubDB.select(db, min_key: {:articles, 0}, max_key: {:articles, nil})This range selection works because our IDs are positive integers, and nil is greater than all numbers, so {:abc, nil} is greater than {:abc, 123}, but smaller than {:bcd, :123}.","ref":"howto.html#store-multiple-collections","title":"How To - Store multiple collections","type":"extras"},{"doc":"CubDB stores its data in a single file with extension .cub, inside the configured data directory. The filename is a hexadecimal value (containing only lowercase letters from a to f and digits) and gets incremented by one upon each compaction. Backing up a database is as simple as copying its current data file (that can be found by calling CubDB.current_db_file/1). Note that, during a compaction, a file with extension .compact is also created: you don't need to copy that file for your backup, as the .cub file already contains all data.To recover from a saved backup, it is sufficient to copy the backed-up .cub file to a directory and start CubDB on that data directory. Make sure that no other .cub file is present in the same directory, and that the filename is a valid hexadecimal number: should more than one .cub files be present in the same data directory, the one with the greatest hexadecimal value is used, and the others are deleted upon the next compaction.","ref":"howto.html#save-and-restore-a-backup","title":"How To - Save and restore a backup","type":"extras"}]